@@ -32,16 +32,11 @@ import java.io.IOException;
 import java.io.StringReader;
 import java.io.StringWriter;
 import java.math.BigInteger;
-import java.security.KeyFactory;
-import java.security.KeyPair;
-import java.security.KeyPairGenerator;
-import java.security.NoSuchAlgorithmException;
-import java.security.PrivateKey;
-import java.security.PublicKey;
-import java.security.SecureRandom;
+import java.security.*;
 import java.security.interfaces.DSAKey;
 import java.security.interfaces.DSAPrivateKey;
 import java.security.interfaces.DSAPublicKey;
+import java.security.spec.DSAPrivateKeySpec;
 import java.security.spec.DSAPublicKeySpec;
 import java.security.spec.InvalidKeySpecException;
 
@@ -61,6 +56,7 @@ import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
+import org.jruby.util.ByteList;
 
 import static org.jruby.ext.openssl.OpenSSL.*;
 import static org.jruby.ext.openssl.impl.PKey.readDSAPrivateKey;
@@ -111,11 +107,28 @@ public class PKeyDSA extends PKey {
     // a public key to be constructed incrementally, as required by the
     // current implementation of Net::SSH.
     // (see net-ssh-1.1.2/lib/net/ssh/transport/ossl/buffer.rb #read_keyblob)
+    private transient volatile BigInteger dsa_x;
     private transient volatile BigInteger dsa_y;
     private transient volatile BigInteger dsa_p;
     private transient volatile BigInteger dsa_q;
     private transient volatile BigInteger dsa_g;
 
+    @Override
+    public IRubyObject initialize_copy(final IRubyObject original) {
+        if (this == original) return this;
+        checkFrozen();
+
+        final PKeyDSA that = (PKeyDSA) original;
+        this.publicKey = that.publicKey;
+        this.privateKey = that.privateKey;
+        this.dsa_x = that.dsa_x;
+        this.dsa_y = that.dsa_y;
+        this.dsa_p = that.dsa_p;
+        this.dsa_q = that.dsa_q;
+        this.dsa_g = that.dsa_g;
+        return this;
+    }
+
     @Override
     public PublicKey getPublicKey() { return publicKey; }
 
@@ -127,28 +140,29 @@ public class PKeyDSA extends PKey {
 
     @JRubyMethod(name = "generate", meta = true)
     public static IRubyObject generate(IRubyObject self, IRubyObject arg) {
-        final int keysize = RubyNumeric.fix2int(arg);
-        PKeyDSA dsa = new PKeyDSA(self.getRuntime(), (RubyClass) self);
-        dsaGenerate(dsa, keysize);
-        return dsa;
+        final Ruby runtime = self.getRuntime();
+        final int keySize = RubyNumeric.fix2int(arg);
+        return dsaGenerate(runtime, new PKeyDSA(runtime, (RubyClass) self), keySize);
     }
 
     /*
      * c: dsa_generate
      */
-    private static void dsaGenerate(PKeyDSA dsa, int keysize) throws RaiseException {
+    private static PKeyDSA dsaGenerate(final Ruby runtime,
+        PKeyDSA dsa, int keySize) throws RaiseException {
         try {
             KeyPairGenerator gen = SecurityHelper.getKeyPairGenerator("DSA");
-            gen.initialize(keysize, new SecureRandom());
+            gen.initialize(keySize, getSecureRandom(runtime));
             KeyPair pair = gen.generateKeyPair();
             dsa.privateKey = (DSAPrivateKey) pair.getPrivate();
             dsa.publicKey = (DSAPublicKey) pair.getPublic();
+            return dsa;
         }
         catch (NoSuchAlgorithmException e) {
-            throw newDSAError(dsa.getRuntime(), e.getMessage());
+            throw newDSAError(runtime, e.getMessage());
         }
         catch (RuntimeException e) {
-            throw newDSAError(dsa.getRuntime(), e.getMessage(), e);
+            throw newDSAError(runtime, e.getMessage(), e);
         }
     }
 
@@ -170,41 +184,50 @@ public class PKeyDSA extends PKey {
         if ( args.length > 1 ) pass = args[1];
 
         if ( arg instanceof RubyFixnum ) {
-            int keysize = RubyNumeric.fix2int((RubyFixnum) arg);
-            dsaGenerate(this, keysize); return this;
+            int keySize = RubyNumeric.fix2int((RubyFixnum) arg);
+            return dsaGenerate(context.runtime, this, keySize);
         }
 
         final char[] passwd = password(pass);
         final RubyString str = readInitArg(context, arg);
+        final String strJava = str.toString();
 
         Object key = null;
         final KeyFactory dsaFactory;
         try {
             dsaFactory = SecurityHelper.getKeyFactory("DSA");
-        } catch (NoSuchAlgorithmException e) {
+        }
+        catch (NoSuchAlgorithmException e) {
             throw runtime.newRuntimeError("unsupported key algorithm (DSA)");
-        } catch (RuntimeException e) {
+        }
+        catch (RuntimeException e) {
             throw runtime.newRuntimeError("unsupported key algorithm (DSA) " + e);
         }
         // TODO: ugly NoClassDefFoundError catching for no BC env. How can we remove this?
         boolean noClassDef = false;
         if ( key == null && ! noClassDef ) { // PEM_read_bio_DSAPrivateKey
             try {
-                key = PEMInputOutput.readDSAPrivateKey(new StringReader(str.toString()), passwd);
+                key = readPrivateKey(strJava, passwd);
             }
             catch (NoClassDefFoundError e) { noClassDef = true; debugStackTrace(runtime, e); }
+            catch (PEMInputOutput.PasswordRequiredException retry) {
+                if ( ttySTDIN(context) ) {
+                    try { key = readPrivateKey(strJava, passwordPrompt(context)); }
+                    catch (Exception e) { debugStackTrace(runtime, e); }
+                }
+            }
             catch (Exception e) { debugStackTrace(runtime, e); }
         }
         if ( key == null && ! noClassDef ) { // PEM_read_bio_DSAPublicKey
             try {
-                key = PEMInputOutput.readDSAPublicKey(new StringReader(str.toString()), passwd);
+                key = PEMInputOutput.readDSAPublicKey(new StringReader(strJava), passwd);
             }
             catch (NoClassDefFoundError e) { noClassDef = true; debugStackTrace(runtime, e); }
             catch (Exception e) { debugStackTrace(runtime, e); }
         }
         if ( key == null && ! noClassDef ) { // PEM_read_bio_DSA_PUBKEY
             try {
-                key = PEMInputOutput.readDSAPubKey(new StringReader(str.toString()));
+                key = PEMInputOutput.readDSAPubKey(new StringReader(strJava));
             }
             catch (NoClassDefFoundError e) { noClassDef = true; debugStackTrace(runtime, e); }
             catch (Exception e) { debugStackTrace(runtime, e); }
@@ -263,6 +286,11 @@ public class PKeyDSA extends PKey {
         return this;
     }
 
+    //private static Object readPrivateKey(final RubyString str, final char[] passwd)
+    //    throws PEMInputOutput.PasswordRequiredException, IOException {
+    //    return PEMInputOutput.readDSAPrivateKey(new StringReader(str.toString()), passwd);
+    //}
+
     @JRubyMethod(name = "public?")
     public RubyBoolean public_p() {
         return publicKey != null ? getRuntime().getTrue() : getRuntime().getFalse();
@@ -343,31 +371,63 @@ public class PKeyDSA extends PKey {
         }
     }
 
-    @JRubyMethod
-    public IRubyObject syssign(IRubyObject arg) {
-        // TODO
-        return getRuntime().getNil();
+    @JRubyMethod // ossl_dsa_sign
+    public IRubyObject syssign(IRubyObject data) {
+        final Ruby runtime = getRuntime();
+
+        DSAPrivateKey privateKey;
+        if ((privateKey = this.privateKey) == null) {
+            throw newDSAError(runtime, "Private DSA key needed!");
+        }
+
+        try {
+            ByteList sign = sign("NONEwithDSA", privateKey, data.convertToString().getByteList()); // DSS1
+            return RubyString.newString(runtime, sign);
+        }
+        catch (GeneralSecurityException ex) {
+            throw newDSAError(runtime, ex.getMessage());
+        }
     }
 
-    @JRubyMethod
-    public IRubyObject sysverify(IRubyObject arg, IRubyObject arg2) {
-        // TODO
-        return getRuntime().getNil();
+    @JRubyMethod // ossl_dsa_verify
+    public IRubyObject sysverify(IRubyObject data, IRubyObject sign) {
+        final Ruby runtime = getRuntime();
+        ByteList sigBytes = convertToString(runtime, sign, "OpenSSL::PKey::DSAError", "invalid signature").getByteList();
+        ByteList dataBytes = convertToString(runtime, data, "OpenSSL::PKey::DSAError", "invalid data").getByteList();
+        try {
+            return runtime.newBoolean( verify("NONEwithDSA", getPublicKey(), dataBytes, sigBytes) );
+        }
+        catch (NoSuchAlgorithmException e) {
+            throw newDSAError(runtime, e.getMessage());
+        }
+        catch (SignatureException e) {
+            throw newDSAError(runtime, "invalid signature");
+        }
+        catch (InvalidKeyException e) {
+            throw newDSAError(runtime, "invalid key");
+        }
+    }
+
+    private DSAKey getDsaKey() {
+        DSAKey result;
+        return (result = publicKey) != null ? result : privateKey;
+    }
+
+    private IRubyObject toBN(BigInteger value) {
+        return value == null ? getRuntime().getNil() : BN.newBN(getRuntime(), value);
+    }
+
+    private synchronized BigInteger getP() {
+        DSAKey key = getDsaKey();
+        if (key != null) {
+            return key.getParams().getP();
+        }
+        return dsa_p;
     }
 
     @JRubyMethod(name = "p")
-    public synchronized IRubyObject get_p() {
-        // FIXME: return only for public?
-        DSAKey key; BigInteger param;
-        if ((key = this.publicKey) != null || (key = this.privateKey) != null) {
-            if ((param = key.getParams().getP()) != null) {
-                return BN.newBN(getRuntime(), param);
-            }
-        }
-        else if (dsa_p != null) {
-            return BN.newBN(getRuntime(), dsa_p);
-        }
-        return getRuntime().getNil();
+    public IRubyObject get_p() {
+        return toBN(getP());
     }
 
     @JRubyMethod(name = "p=")
@@ -375,19 +435,17 @@ public class PKeyDSA extends PKey {
         return setKeySpecComponent(SPEC_P, p);
     }
 
+    private synchronized BigInteger getQ() {
+        DSAKey key = getDsaKey();
+        if (key != null) {
+            return key.getParams().getQ();
+        }
+        return dsa_q;
+    }
+
     @JRubyMethod(name = "q")
-    public synchronized IRubyObject get_q() {
-        // FIXME: return only for public?
-        DSAKey key; BigInteger param;
-        if ((key = this.publicKey) != null || (key = this.privateKey) != null) {
-            if ((param = key.getParams().getQ()) != null) {
-                return BN.newBN(getRuntime(), param);
-            }
-        }
-        else if (dsa_q != null) {
-            return BN.newBN(getRuntime(), dsa_q);
-        }
-        return getRuntime().getNil();
+    public IRubyObject get_q() {
+        return toBN(getQ());
     }
 
     @JRubyMethod(name = "q=")
@@ -395,19 +453,17 @@ public class PKeyDSA extends PKey {
         return setKeySpecComponent(SPEC_Q, q);
     }
 
+    private synchronized BigInteger getG() {
+        DSAKey key = getDsaKey();
+        if (key != null) {
+            return key.getParams().getG();
+        }
+        return dsa_g;
+    }
+
     @JRubyMethod(name = "g")
-    public synchronized IRubyObject get_g() {
-        // FIXME: return only for public?
-        DSAKey key; BigInteger param;
-        if ((key = this.publicKey) != null || (key = this.privateKey) != null) {
-            if ((param = key.getParams().getG()) != null) {
-                return BN.newBN(getRuntime(), param);
-            }
-        }
-        else if (dsa_g != null) {
-            return BN.newBN(getRuntime(), dsa_g);
-        }
-        return getRuntime().getNil();
+    public IRubyObject get_g() {
+        return toBN(getG());
     }
 
     @JRubyMethod(name = "g=")
@@ -415,25 +471,27 @@ public class PKeyDSA extends PKey {
         return setKeySpecComponent(SPEC_G, g);
     }
 
-    @JRubyMethod(name = "pub_key")
-    public synchronized IRubyObject get_pub_key() {
-        DSAPublicKey key;
-        if ( ( key = this.publicKey ) != null ) {
-            return BN.newBN(getRuntime(), key.getY());
-        }
-        else if (dsa_y != null) {
-            return BN.newBN(getRuntime(), dsa_y);
-        }
-        return getRuntime().getNil();
-    }
-
     @JRubyMethod(name = "priv_key")
     public synchronized IRubyObject get_priv_key() {
         DSAPrivateKey key;
         if ((key = this.privateKey) != null) {
-            return BN.newBN(getRuntime(), key.getX());
+            return toBN(key.getX());
         }
-        return getRuntime().getNil();
+        return toBN(dsa_x);
+    }
+
+    @JRubyMethod(name = "priv_key=")
+    public synchronized IRubyObject set_priv_key(IRubyObject priv_key) {
+        return setKeySpecComponent(SPEC_X, priv_key);
+    }
+
+    @JRubyMethod(name = "pub_key")
+    public synchronized IRubyObject get_pub_key() {
+        DSAPublicKey key;
+        if ( ( key = this.publicKey ) != null ) {
+            return toBN(key.getY());
+        }
+        return toBN(dsa_y);
     }
 
     @JRubyMethod(name = "pub_key=")
@@ -445,15 +503,38 @@ public class PKeyDSA extends PKey {
         final BigInteger val = BN.getBigInteger(value);
 
         switch (index) {
+            case SPEC_X: this.dsa_x = val; break;
             case SPEC_Y: this.dsa_y = val; break;
             case SPEC_P: this.dsa_p = val; break;
             case SPEC_Q: this.dsa_q = val; break;
             case SPEC_G: this.dsa_g = val; break;
         }
 
-        if ( dsa_y != null && dsa_p != null && dsa_q != null && dsa_g != null ) {
-            // we now have all components. create the key :
-            DSAPublicKeySpec spec = new DSAPublicKeySpec(dsa_y, dsa_p, dsa_q, dsa_g);
+        // Don't access the dsa_p, dsa_q and dsa_g fields directly. They may
+        // have already been consumed and cleared.
+        BigInteger _dsa_p = getP();
+        BigInteger _dsa_q = getQ();
+        BigInteger _dsa_g = getG();
+
+        if ( dsa_x != null && _dsa_p != null && _dsa_q != null && _dsa_g != null ) {
+            // we now have all private key components. create the key :
+            DSAPrivateKeySpec spec = new DSAPrivateKeySpec(dsa_x, _dsa_p, _dsa_q, _dsa_g);
+            try {
+                this.privateKey = (DSAPrivateKey) SecurityHelper.getKeyFactory("DSA").generatePrivate(spec);
+            }
+            catch (InvalidKeySpecException e) {
+                throw newDSAError(getRuntime(), "invalid keyspec", e);
+            }
+            catch (NoSuchAlgorithmException e) {
+                throw newDSAError(getRuntime(), "unsupported key algorithm (DSA)", e);
+            }
+            // clear out the specValues
+            this.dsa_x = this.dsa_p = this.dsa_q = this.dsa_g = null;
+        }
+
+        if ( dsa_y != null && _dsa_p != null && _dsa_q != null && _dsa_g != null ) {
+            // we now have all public key components. create the key :
+            DSAPublicKeySpec spec = new DSAPublicKeySpec(dsa_y, _dsa_p, _dsa_q, _dsa_g);
             try {
                 this.publicKey = (DSAPublicKey) SecurityHelper.getKeyFactory("DSA").generatePublic(spec);
             }
@@ -470,10 +551,11 @@ public class PKeyDSA extends PKey {
         return value;
     }
 
-    private static final int SPEC_Y = 0;
-    private static final int SPEC_P = 1;
-    private static final int SPEC_Q = 2;
-    private static final int SPEC_G = 3;
+    private static final int SPEC_X = 0;
+    private static final int SPEC_Y = 1;
+    private static final int SPEC_P = 2;
+    private static final int SPEC_Q = 3;
+    private static final int SPEC_G = 4;
 
     public static RaiseException newDSAError(Ruby runtime, String message) {
         return Utils.newError(runtime, _PKey(runtime).getClass("DSAError"), message);
