@@ -71,7 +71,6 @@ import org.jruby.ext.openssl.impl.CipherSpec;
 import org.jruby.ext.openssl.x509store.PEMInputOutput;
 import static org.jruby.ext.openssl.OpenSSL.*;
 import static org.jruby.ext.openssl.PKey._PKey;
-import static org.jruby.ext.openssl.PKey.cipherSpec;
 import static org.jruby.ext.openssl.impl.PKey.readRSAPrivateKey;
 import static org.jruby.ext.openssl.impl.PKey.readRSAPublicKey;
 import static org.jruby.ext.openssl.impl.PKey.toDerRSAKey;
@@ -107,6 +106,10 @@ public class PKeyRSA extends PKey {
         return Utils.newError(runtime, _PKey(runtime).getClass("RSAError"), message);
     }
 
+    static RaiseException newRSAError(Ruby runtime, Throwable cause) {
+        return Utils.newError(runtime, _PKey(runtime).getClass("RSAError"), cause.getMessage(), cause);
+    }
+
     public PKeyRSA(Ruby runtime, RubyClass type) {
         super(runtime, type);
     }
@@ -138,6 +141,25 @@ public class PKeyRSA extends PKey {
     private transient volatile BigInteger rsa_dmq1;
     private transient volatile BigInteger rsa_iqmp;
 
+    @Override
+    public IRubyObject initialize_copy(final IRubyObject original) {
+        if (this == original) return this;
+        checkFrozen();
+
+        final PKeyRSA that = (PKeyRSA) original;
+        this.publicKey = that.publicKey;
+        this.privateKey = that.privateKey;
+        this.rsa_e = that.rsa_e;
+        this.rsa_n = that.rsa_n;
+        this.rsa_d = that.rsa_d;
+        this.rsa_p = that.rsa_p;
+        this.rsa_q = that.rsa_q;
+        this.rsa_dmp1 = that.rsa_dmp1;
+        this.rsa_dmq1 = that.rsa_dmq1;
+        this.rsa_iqmp = that.rsa_iqmp;
+        return this;
+    }
+
     @Override
     public PublicKey getPublicKey() { return publicKey; }
 
@@ -149,44 +171,45 @@ public class PKeyRSA extends PKey {
 
     @JRubyMethod(name = "generate", meta = true, rest = true)
     public static IRubyObject generate(IRubyObject self, IRubyObject[] args) {
+        final Ruby runtime = self.getRuntime();
         BigInteger exp = RSAKeyGenParameterSpec.F4;
-        if ( Arity.checkArgumentCount(self.getRuntime(), args, 1, 2) == 2 ) {
+        if ( Arity.checkArgumentCount(runtime, args, 1, 2) == 2 ) {
             if (args[1] instanceof RubyFixnum) {
                 exp = BigInteger.valueOf(RubyNumeric.num2long(args[1]));
             } else {
                 exp = ((RubyBignum) args[1]).getValue();
             }
         }
-        int keysize = RubyNumeric.fix2int(args[0]);
-        PKeyRSA rsa = new PKeyRSA(self.getRuntime(), (RubyClass) self);
-        rsaGenerate(rsa, keysize, exp);
-        return rsa;
+        final int keySize = RubyNumeric.fix2int(args[0]);
+        return rsaGenerate(runtime, new PKeyRSA(runtime, (RubyClass) self), keySize, exp);
     }
 
     /*
      * c: rsa_generate
      */
-    private static void rsaGenerate(PKeyRSA rsa, int keysize, BigInteger exp) throws RaiseException {
+    private static PKeyRSA rsaGenerate(final Ruby runtime,
+        PKeyRSA rsa, int keySize, BigInteger exp) throws RaiseException {
         try {
             KeyPairGenerator gen = SecurityHelper.getKeyPairGenerator("RSA");
             if ( "IBMJCEFIPS".equals( gen.getProvider().getName() ) ) {
-                gen.initialize(keysize); // IBMJCEFIPS does not support parameters
+                gen.initialize(keySize); // IBMJCEFIPS does not support parameters
             } else {
-                gen.initialize(new RSAKeyGenParameterSpec(keysize, exp), new SecureRandom());
+                gen.initialize(new RSAKeyGenParameterSpec(keySize, exp), getSecureRandom(runtime));
             }
             KeyPair pair = gen.generateKeyPair();
             rsa.privateKey = (RSAPrivateCrtKey) pair.getPrivate();
             rsa.publicKey = (RSAPublicKey) pair.getPublic();
         }
         catch (NoSuchAlgorithmException e) {
-            throw newRSAError(rsa.getRuntime(), e.getMessage());
+            throw newRSAError(runtime, e.getMessage());
         }
         catch (InvalidAlgorithmParameterException e) {
-            throw newRSAError(rsa.getRuntime(), e.getMessage());
+            throw newRSAError(runtime, e.getMessage());
         }
         catch (RuntimeException e) {
-            throw newRSAError(rsa.getRuntime(), e.getMessage());
+            throw newRSAError(rsa.getRuntime(), e);
         }
+        return rsa;
     }
 
     static PKeyRSA newInstance(final Ruby runtime, final PublicKey publicKey) {
@@ -207,45 +230,54 @@ public class PKeyRSA extends PKey {
         if ( args.length > 1 ) pass = args[1];
 
         if ( arg instanceof RubyFixnum ) {
-            int keysize = RubyNumeric.fix2int((RubyFixnum) arg);
+            int keySize = RubyNumeric.fix2int((RubyFixnum) arg);
             BigInteger exp = RSAKeyGenParameterSpec.F4;
-            if (null != pass && !pass.isNil()) {
+            if ( pass != null && ! pass.isNil() ) {
                 exp = BigInteger.valueOf(RubyNumeric.num2long(pass));
             }
-            rsaGenerate(this, keysize, exp); return this;
+            return rsaGenerate(runtime, this, keySize, exp);
         }
 
         final char[] passwd = password(pass);
         final RubyString str = readInitArg(context, arg);
+        final String strJava = str.toString();
 
         Object key = null;
         final KeyFactory rsaFactory;
         try {
             rsaFactory = SecurityHelper.getKeyFactory("RSA");
-        } catch (NoSuchAlgorithmException e) {
+        }
+        catch (NoSuchAlgorithmException e) {
             throw runtime.newRuntimeError("unsupported key algorithm (RSA)");
-        } catch (RuntimeException e) {
+        }
+        catch (RuntimeException e) {
             throw runtime.newRuntimeError("unsupported key algorithm (RSA) " + e);
         }
         // TODO: ugly NoClassDefFoundError catching for no BC env. How can we remove this?
         boolean noClassDef = false;
         if ( key == null && ! noClassDef ) { // PEM_read_bio_RSAPrivateKey
             try {
-                key = PEMInputOutput.readPrivateKey(new StringReader(str.toString()), passwd);
+                key = readPrivateKey(strJava, passwd);
             }
             catch (NoClassDefFoundError e) { noClassDef = true; debugStackTrace(runtime, e); }
+            catch (PEMInputOutput.PasswordRequiredException retry) {
+                if ( ttySTDIN(context) ) {
+                    try { key = readPrivateKey(strJava, passwordPrompt(context)); }
+                    catch (Exception e) { debugStackTrace(runtime, e); }
+                }
+            }
             catch (Exception e) { debugStackTrace(runtime, e); }
         }
         if ( key == null && ! noClassDef )  { // PEM_read_bio_RSAPublicKey
             try {
-                key = PEMInputOutput.readRSAPublicKey(new StringReader(str.toString()), passwd);
+                key = PEMInputOutput.readRSAPublicKey(new StringReader(strJava), passwd);
             }
             catch (NoClassDefFoundError e) { noClassDef = true; debugStackTrace(runtime, e); }
             catch (Exception e) { debugStackTrace(runtime, e); }
         }
         if ( key == null && ! noClassDef ) { // PEM_read_bio_RSA_PUBKEY
             try {
-                key = PEMInputOutput.readRSAPubKey(new StringReader(str.toString()));
+                key = PEMInputOutput.readRSAPubKey(new StringReader(strJava));
             }
             catch (NoClassDefFoundError e) { noClassDef = true; debugStackTrace(runtime, e); }
             catch (Exception e) { debugStackTrace(runtime, e); }
@@ -313,9 +345,14 @@ public class PKeyRSA extends PKey {
         return publicKey != null ? getRuntime().getTrue() : getRuntime().getFalse();
     }
 
+    @Override
+    public boolean isPrivateKey() {
+        return privateKey != null;
+    }
+
     @JRubyMethod(name = "private?")
     public RubyBoolean private_p() {
-        return privateKey != null ? getRuntime().getTrue() : getRuntime().getFalse();
+        return getRuntime().newBoolean(isPrivateKey());
     }
 
     @Override
@@ -371,7 +408,7 @@ public class PKeyRSA extends PKey {
         StringBuilder result = new StringBuilder();
         if (privateKey != null) {
             int len = privateKey.getModulus().bitLength();
-            result.append("Private-Key: (").append(len).append(" bit)").append("\n");
+            result.append("Private-Key: (").append(len).append(" bit)").append('\n');
             result.append("modulus:");
             addSplittedAndFormatted(result, privateKey.getModulus());
             result.append("publicExponent: ").append(privateKey.getPublicExponent()).append(" (0x").append(privateKey.getPublicExponent().toString(16)).append(")\n");
@@ -393,7 +430,7 @@ public class PKeyRSA extends PKey {
             addSplittedAndFormatted(result, publicKey.getModulus());
             result.append("Exponent: ").append(publicKey.getPublicExponent()).append(" (0x").append(publicKey.getPublicExponent().toString(16)).append(")\n");
         }
-        return getRuntime().newString(result.toString());
+        return RubyString.newString(getRuntime(), result);
     }
 
     @Override
@@ -427,7 +464,7 @@ public class PKeyRSA extends PKey {
 
     private String getPadding(final int padding) {
         if ( padding < 1 || padding > 4 ) {
-            throw newRSAError(getRuntime(), null);
+            throw newRSAError(getRuntime(), "");
         }
         // BC accepts "/NONE/*" but SunJCE doesn't. use "/ECB/*"
         String p = "/ECB/PKCS1Padding";
@@ -447,7 +484,7 @@ public class PKeyRSA extends PKey {
         if ( Arity.checkArgumentCount(context.runtime, args, 1, 2) == 2 && ! args[1].isNil() ) {
             padding = RubyNumeric.fix2int(args[1]);
         }
-        if ( privateKey == null ) throw newRSAError(context.runtime, "private key needed.");
+        if ( privateKey == null ) throw newRSAError(context.runtime, "incomplete RSA");
         return doCipherRSA(context.runtime, args[0], padding, ENCRYPT_MODE, privateKey);
     }
 
@@ -457,7 +494,7 @@ public class PKeyRSA extends PKey {
         if ( Arity.checkArgumentCount(context.runtime, args, 1, 2) == 2 && ! args[1].isNil())  {
             padding = RubyNumeric.fix2int(args[1]);
         }
-        if ( privateKey == null ) throw newRSAError(context.runtime, "private key needed.");
+        if ( privateKey == null ) throw newRSAError(context.runtime, "incomplete RSA");
         return doCipherRSA(context.runtime, args[0], padding, DECRYPT_MODE, privateKey);
     }
 
@@ -467,6 +504,7 @@ public class PKeyRSA extends PKey {
         if ( Arity.checkArgumentCount(context.runtime, args, 1, 2) == 2 && ! args[1].isNil())  {
             padding = RubyNumeric.fix2int(args[1]);
         }
+        if ( publicKey == null ) throw newRSAError(context.runtime, "incomplete RSA");
         return doCipherRSA(context.runtime, args[0], padding, ENCRYPT_MODE, publicKey);
     }
 
@@ -476,6 +514,7 @@ public class PKeyRSA extends PKey {
         if ( Arity.checkArgumentCount(context.runtime, args, 1, 2) == 2 && ! args[1].isNil() ) {
             padding = RubyNumeric.fix2int(args[1]);
         }
+        if ( publicKey == null ) throw newRSAError(context.runtime, "incomplete RSA");
         return doCipherRSA(context.runtime, args[0], padding, DECRYPT_MODE, publicKey);
     }
 
@@ -531,7 +570,7 @@ public class PKeyRSA extends PKey {
         if ( privateKey != null ) {
             throw newRSAError(context.runtime, "illegal modification");
         }
-        rsa_dmp1 = BN.getBigInteger(value);
+        rsa_dmp1 = BN.asBigInteger(value);
         generatePrivateKeyIfParams(context);
         return value;
     }
@@ -541,7 +580,7 @@ public class PKeyRSA extends PKey {
         if ( privateKey != null ) {
             throw newRSAError(context.runtime, "illegal modification");
         }
-        rsa_dmq1 = BN.getBigInteger(value);
+        rsa_dmq1 = BN.asBigInteger(value);
         generatePrivateKeyIfParams(context);
         return value;
     }
@@ -551,7 +590,7 @@ public class PKeyRSA extends PKey {
         if ( privateKey != null ) {
             throw newRSAError(context.runtime, "illegal modification");
         }
-        rsa_iqmp = BN.getBigInteger(value);
+        rsa_iqmp = BN.asBigInteger(value);
         generatePrivateKeyIfParams(context);
         return value;
     }
@@ -640,17 +679,19 @@ public class PKeyRSA extends PKey {
         return getRuntime().getNil();
     }
 
+    private BigInteger getPublicExponent() {
+        if (publicKey != null) {
+            return publicKey.getPublicExponent();
+        } else if (privateKey != null) {
+            return privateKey.getPublicExponent();
+        } else {
+            return rsa_e;
+        }
+    }
+
     @JRubyMethod(name="e")
     public synchronized IRubyObject get_e() {
-        RSAPublicKey key;
-        BigInteger e;
-        if ((key = publicKey) != null) {
-            e = key.getPublicExponent();
-        } else if(privateKey != null) {
-            e = privateKey.getPublicExponent();
-        } else {
-            e = rsa_e;
-        }
+        BigInteger e = getPublicExponent();
         if (e != null) {
             return BN.newBN(getRuntime(), e);
         }
@@ -671,17 +712,19 @@ public class PKeyRSA extends PKey {
         return value;
     }
 
+    private BigInteger getModulus() {
+        if (publicKey != null) {
+            return publicKey.getModulus();
+        } else if (privateKey != null) {
+            return privateKey.getModulus();
+        } else {
+            return rsa_n;
+        }
+    }
+
     @JRubyMethod(name="n")
     public synchronized IRubyObject get_n() {
-        RSAPublicKey key;
-        BigInteger n;
-        if ((key = publicKey) != null) {
-            n = key.getModulus();
-        } else if(privateKey != null) {
-            n = privateKey.getModulus();
-        } else {
-            n = rsa_n;
-        }
+        BigInteger n = getModulus();
         if (n != null) {
             return BN.newBN(getRuntime(), n);
         }
@@ -707,8 +750,12 @@ public class PKeyRSA extends PKey {
 
         if ( publicKey != null ) throw newRSAError(runtime, "illegal modification");
 
-        BigInteger e, n;
-        if ( (e = rsa_e) != null && (n = rsa_n) != null ) {
+        // Don't access the rsa_n and rsa_e fields directly. They may have
+        // already been consumed and cleared by generatePrivateKeyIfParams.
+        BigInteger _rsa_n = getModulus();
+        BigInteger _rsa_e = getPublicExponent();
+
+        if (_rsa_n != null && _rsa_e != null) {
             final KeyFactory rsaFactory;
             try {
                 rsaFactory = SecurityHelper.getKeyFactory("RSA");
@@ -718,7 +765,7 @@ public class PKeyRSA extends PKey {
             }
 
             try {
-                publicKey = (RSAPublicKey) rsaFactory.generatePublic(new RSAPublicKeySpec(n, e));
+                publicKey = (RSAPublicKey) rsaFactory.generatePublic(new RSAPublicKeySpec(_rsa_n, _rsa_e));
             }
             catch (InvalidKeySpecException ex) {
                 throw newRSAError(runtime, "invalid parameters");
@@ -733,7 +780,12 @@ public class PKeyRSA extends PKey {
 
         if ( privateKey != null ) throw newRSAError(runtime, "illegal modification");
 
-        if (rsa_e != null && rsa_n != null && rsa_p != null && rsa_q != null && rsa_d != null && rsa_dmp1 != null && rsa_dmq1 != null && rsa_iqmp != null) {
+        // Don't access the rsa_n and rsa_e fields directly. They may have
+        // already been consumed and cleared by generatePublicKeyIfParams.
+        BigInteger _rsa_n = getModulus();
+        BigInteger _rsa_e = getPublicExponent();
+
+        if (_rsa_n != null && _rsa_e != null && rsa_p != null && rsa_q != null && rsa_d != null && rsa_dmp1 != null && rsa_dmq1 != null && rsa_iqmp != null) {
             final KeyFactory rsaFactory;
             try {
                 rsaFactory = SecurityHelper.getKeyFactory("RSA");
@@ -744,7 +796,7 @@ public class PKeyRSA extends PKey {
 
             try {
                 privateKey = (RSAPrivateCrtKey) rsaFactory.generatePrivate(
-                    new RSAPrivateCrtKeySpec(rsa_n, rsa_e, rsa_d, rsa_p, rsa_q, rsa_dmp1, rsa_dmq1, rsa_iqmp)
+                    new RSAPrivateCrtKeySpec(_rsa_n, _rsa_e, rsa_d, rsa_p, rsa_q, rsa_dmp1, rsa_dmq1, rsa_iqmp)
                 );
             }
             catch (InvalidKeySpecException e) {
