@@ -38,6 +38,7 @@ import org.jruby.internal.runtime.methods.DynamicMethod;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
+import org.jruby.util.SafePropertyAccessor;
 
 /**
  * @author <a href="mailto:ola.bini@ki.se">Ola Bini</a>
@@ -170,20 +171,36 @@ public class SSL {
     }
 
     public static RaiseException newSSLErrorWaitReadable(Ruby runtime, String message) {
-        return newCustomSSLError(runtime, "SSLErrorWaitReadable", message);
+        return newWaitSSLError(runtime, "SSLErrorWaitReadable", message);
     }
 
     public static RaiseException newSSLErrorWaitWritable(Ruby runtime, String message) {
-        return newCustomSSLError(runtime, "SSLErrorWaitWritable", message);
+        return newWaitSSLError(runtime, "SSLErrorWaitWritable", message);
     }
 
-    private static RaiseException newCustomSSLError(final Ruby runtime, final String name,
+    // -Djruby.openssl.ssl.error_wait_nonblock.backtrace=false disables backtrace for WaitReadable/Writable
+    private static final boolean waitErrorBacktrace;
+
+    static {
+        String backtrace = SafePropertyAccessor.getProperty("jruby.openssl.ssl.error_wait_nonblock.backtrace");
+        if (backtrace == null) {
+            // default to JRuby's Option<Boolean> ERRNO_BACKTRACE =
+            // ... "Generate backtraces for heavily-used Errno exceptions (EAGAIN)."
+            backtrace = SafePropertyAccessor.getProperty("jruby.errno.backtrace", "false");
+        }
+        waitErrorBacktrace = Boolean.parseBoolean(backtrace);
+    }
+
+    private static RaiseException newWaitSSLError(final Ruby runtime, final String name,
         final String message) {
         RubyClass errorClass = _SSL(runtime).getClass(name);
         if ( errorClass == null ) { // < Ruby 2.0
             errorClass = _SSL(runtime).getClass("SSLError"); // fallback
         }
-        return Utils.newError(runtime, errorClass, message, false);
+        if ( waitErrorBacktrace ) {
+            return Utils.newError(runtime, errorClass, message, false);
+        }
+        return Utils.newErrorWithoutTrace(runtime, errorClass, message, false);
     }
 
     static RubyModule _SSL(final Ruby runtime) {
