@@ -242,6 +242,7 @@ public class SSLSocket extends RubyObject {
                 handshakeStatus = engine.getHandshakeStatus();
                 initialHandshake = true;
             }
+            callRenegotiationCallback(context);
             final IRubyObject ex = doHandshake(blocking, exception);
             if ( ex != null ) return ex; // :wait_readable | :wait_writable
         }
@@ -325,6 +326,7 @@ public class SSLSocket extends RubyObject {
                 handshakeStatus = engine.getHandshakeStatus();
                 initialHandshake = true;
             }
+            callRenegotiationCallback(context);
             final IRubyObject ex = doHandshake(blocking, exception);
             if ( ex != null ) return ex; // :wait_readable | :wait_writable
         }
@@ -592,6 +594,18 @@ public class SSLSocket extends RubyObject {
     private void finishInitialHandshake() {
         initialHandshake = false;
     }
+    
+    private void callRenegotiationCallback(final ThreadContext context) throws RaiseException {
+        IRubyObject renegotiationCallback = sslContext.getInstanceVariable("@renegotiation_cb");
+        if(renegotiationCallback == null || renegotiationCallback.isNil()) {
+            return;
+        }
+        else {
+            // the return of the Proc is not important
+            // Can throw ruby exception to "disallow" renegotiations
+            renegotiationCallback.callMethod(context, "call", this);
+        }    
+    }
 
     public int write(ByteBuffer src, boolean blocking) throws SSLException, IOException {
         if ( initialHandshake ) {
