@@ -64,6 +64,8 @@ import java.util.concurrent.ConcurrentHashMap;
 
 import javax.crypto.Cipher;
 import javax.crypto.CipherSpi;
+import javax.crypto.KeyAgreement;
+import javax.crypto.KeyAgreementSpi;
 import javax.crypto.KeyGenerator;
 import javax.crypto.KeyGeneratorSpi;
 import javax.crypto.Mac;
@@ -97,17 +99,18 @@ import org.bouncycastle.operator.bc.BcRSAContentVerifierProviderBuilder;
 public abstract class SecurityHelper {
 
     private static String BC_PROVIDER_CLASS = "org.bouncycastle.jce.provider.BouncyCastleProvider";
+    private static String BC_PROVIDER_NAME = "BC";
     static boolean setBouncyCastleProvider = true; // (package access for tests)
     static Provider securityProvider; // 'BC' provider (package access for tests)
-    private static Boolean registerProvider = null;
-    private static final Map<String, Class> implEngines = new ConcurrentHashMap<String, Class>(16, 0.75f, 1);
+    private static volatile Boolean registerProvider = null;
+    static final Map<String, Class> implEngines = new ConcurrentHashMap<String, Class>(16, 0.75f, 1);
 
     /**
      * inject under a given name a cipher. also ensures that the registered
      * classes are getting used.
      *
      * @param name the name under which the class gets registered
-     * @param the CipherSpi class
+     * @param clazz the CipherSpi class
      */
     public static void addCipher(String name, Class<? extends CipherSpi> clazz) {
         implEngines.put("Cipher:" + name, clazz);
@@ -118,7 +121,7 @@ public abstract class SecurityHelper {
      * inject under a given name a signature
      *
      * @param name the name under which the class gets registered
-     * @param the SignaturSpi class
+     * @param clazz the SignaturSpi class
      */
     public static void addSignature(String name, Class<? extends SignatureSpi> clazz) {
         implEngines.put("Signature:" + name, clazz);
@@ -136,7 +139,38 @@ public abstract class SecurityHelper {
         return securityProvider;
     }
 
+    static final boolean SPI_ACCESSIBLE;
+
+    static {
+        boolean canSetAccessible = true;
+        if ( OpenSSL.javaVersion9(true) ) {
+            final Provider provider = getSecurityProvider();
+            if ( provider != null ) {
+                try {
+                    // NOTE: some getXxx pieces might still work
+                    // where SPI are returned directly + there's a public <init> e.g. MessageDigest(...)
+                    getCertificateFactory("X.509", provider); // !!! disables EVERYTHING :(
+                }
+                catch (CertificateException ex) {
+                    debugStackTrace(ex);
+                    canSetAccessible = false;
+                }
+                catch (RuntimeException ex) {
+                    debugStackTrace(ex);
+                    // java.lang.reflect.InaccessibleObjectException (extends RuntimeException)
+                    canSetAccessible = false;
+                }
+            }
+        }
+        SPI_ACCESSIBLE = canSetAccessible;
+    }
+
+    static Provider getSecurityProviderIfAccessible() {
+        return SPI_ACCESSIBLE ? getSecurityProvider() : null;
+    }
+
     public static synchronized void setSecurityProvider(final Provider provider) {
+        if ( provider != null ) OpenSSL.debug("using provider: " + provider);
         securityProvider = provider;
     }
 
@@ -152,15 +186,17 @@ public abstract class SecurityHelper {
         return null;
     }
 
-    public static synchronized void setRegisterProvider(boolean register) {
-        registerProvider = Boolean.valueOf(register); doRegisterProvider();
+    public static synchronized void setRegisterProvider(final boolean register) {
+        registerProvider = Boolean.valueOf(register);
+        if ( register ) getSecurityProvider(); // so that securityProvider != null
+        // getSecurityProvider does doRegisterProvider();
     }
 
     static boolean isProviderAvailable(final String name) {
         return Security.getProvider(name) != null;
     }
 
-    static boolean isProviderRegistered() {
+    public static boolean isProviderRegistered() {
         if ( securityProvider == null ) return false;
         return Security.getProvider(securityProvider.getName()) != null;
     }
@@ -168,13 +204,14 @@ public abstract class SecurityHelper {
     private static void doRegisterProvider() {
         if ( registerProvider != null ) {
             synchronized(SecurityHelper.class) {
-                if ( registerProvider != null && registerProvider.booleanValue() ) {
+                final Boolean register = registerProvider;
+                if ( register != null && register.booleanValue() ) {
                     if ( securityProvider != null ) {
                         Security.addProvider(securityProvider);
+                        registerProvider = null;
                     }
                 }
             }
-            registerProvider = null;
         }
     }
 
@@ -184,16 +221,30 @@ public abstract class SecurityHelper {
     public static CertificateFactory getCertificateFactory(final String type)
         throws CertificateException {
         try {
-            final Provider provider = getSecurityProvider();
+            final Provider provider = getSecurityProviderIfAccessible();
             if ( provider != null ) return getCertificateFactory(type, provider);
         }
-        catch (CertificateException e) { }
+        catch (CertificateException e) { debugStackTrace(e); }
         return CertificateFactory.getInstance(type);
     }
 
     static CertificateFactory getCertificateFactory(final String type, final Provider provider)
         throws CertificateException {
-        final CertificateFactorySpi spi = (CertificateFactorySpi) getImplEngine("CertificateFactory", type);
+        final CertificateFactorySpi spi;
+        boolean addedBC = false;
+        synchronized(SecurityHelper.class) {
+            try { // TODO fixed since BC 1.55 (only needed on 1.54) and should be removed eventually ...
+                if (provider.getName().equals(BC_PROVIDER_NAME) && Security.getProvider(BC_PROVIDER_NAME) == null) {
+                    Security.addProvider(provider);
+                    addedBC = true;
+                }
+                spi = (CertificateFactorySpi) getImplEngine("CertificateFactory", type);
+            } finally {
+                if (addedBC) {
+                    Security.removeProvider(BC_PROVIDER_NAME);
+                }
+            }
+        }
         if ( spi == null ) throw new CertificateException(type + " not found");
         return newInstance(CertificateFactory.class,
                 new Class[]{ CertificateFactorySpi.class, Provider.class, String.class },
@@ -207,7 +258,7 @@ public abstract class SecurityHelper {
     public static KeyFactory getKeyFactory(final String algorithm)
         throws NoSuchAlgorithmException {
         try {
-            final Provider provider = getSecurityProvider();
+            final Provider provider = getSecurityProviderIfAccessible();
             if ( provider != null ) return getKeyFactory(algorithm, provider);
         }
         catch (NoSuchAlgorithmException e) { }
@@ -230,7 +281,7 @@ public abstract class SecurityHelper {
     public static KeyPairGenerator getKeyPairGenerator(final String algorithm)
         throws NoSuchAlgorithmException {
         try {
-            final Provider provider = getSecurityProvider();
+            final Provider provider = getSecurityProviderIfAccessible();
             if ( provider != null ) return getKeyPairGenerator(algorithm, provider);
         }
         catch (NoSuchAlgorithmException e) { }
@@ -270,7 +321,7 @@ public abstract class SecurityHelper {
     public static KeyStore getKeyStore(final String type)
         throws KeyStoreException {
         try {
-            final Provider provider = getSecurityProvider();
+            final Provider provider = getSecurityProviderIfAccessible();
             if ( provider != null ) return getKeyStore(type, provider);
         }
         catch (KeyStoreException e) { }
@@ -287,7 +338,7 @@ public abstract class SecurityHelper {
      */
     public static MessageDigest getMessageDigest(final String algorithm) throws NoSuchAlgorithmException {
         try {
-            final Provider provider = getSecurityProvider();
+            final Provider provider = getSecurityProviderIfAccessible();
             if ( provider != null ) return getMessageDigest(algorithm, provider);
         }
         catch (NoSuchAlgorithmException e) { }
@@ -321,7 +372,7 @@ public abstract class SecurityHelper {
 
     public static SecureRandom getSecureRandom() {
         try {
-            final Provider provider = getSecurityProvider();
+            final Provider provider = getSecurityProviderIfAccessible();
             if ( provider != null ) {
                 final String algorithm = getSecureRandomAlgorithm(provider);
                 if ( algorithm != null ) {
@@ -431,22 +482,20 @@ public abstract class SecurityHelper {
 
         }
         try {
-            return newInstance(Cipher.class,
-                new Class[] { CipherSpi.class, Provider.class, String.class },
-                new Object[] { spi, provider, transformation }
-            );
-        }
-        catch( IllegalStateException e ) {
-            // this can be due to trusted check in Cipher constructor
-            if (e.getCause().getClass() == NullPointerException.class) {
-                Cipher cipher = newInstance(Cipher.class,
+            // this constructor does not verify the provider
+            Cipher cipher = newInstance(Cipher.class,
                     new Class[] { CipherSpi.class, String.class },
                     new Object[] { spi, transformation }
-                );
-                setField(cipher, Cipher.class, "provider", provider);
-                return cipher;
-            }
-            throw e;
+            );
+            setField(cipher, Cipher.class, "provider", provider);
+            return cipher;
+        }
+        catch( Exception e ) {
+            // this constructor does verify the provider which might fail
+            return newInstance(Cipher.class,
+                    new Class[] { CipherSpi.class, Provider.class, String.class },
+                    new Object[] { spi, provider, transformation }
+            );
         }
     }
 
@@ -455,7 +504,7 @@ public abstract class SecurityHelper {
      */
     public static Signature getSignature(final String algorithm) throws NoSuchAlgorithmException {
         try {
-            final Provider provider = getSecurityProvider();
+            final Provider provider = getSecurityProviderIfAccessible();
             if ( provider != null ) return getSignature(algorithm, provider);
         }
         catch (NoSuchAlgorithmException e) { }
@@ -491,7 +540,7 @@ public abstract class SecurityHelper {
      */
     public static Mac getMac(final String algorithm) throws NoSuchAlgorithmException {
         Mac mac = null;
-        final Provider provider = getSecurityProvider();
+        final Provider provider = getSecurityProviderIfAccessible();
         if ( provider != null ) {
             mac = getMac(algorithm, provider, true);
         }
@@ -522,7 +571,7 @@ public abstract class SecurityHelper {
      */
     public static KeyGenerator getKeyGenerator(final String algorithm) throws NoSuchAlgorithmException {
         try {
-            final Provider provider = getSecurityProvider();
+            final Provider provider = getSecurityProviderIfAccessible();
             if ( provider != null ) return getKeyGenerator(algorithm, provider);
         }
         catch (NoSuchAlgorithmException e) { }
@@ -541,12 +590,36 @@ public abstract class SecurityHelper {
         );
     }
 
+    /**
+     * @note code calling this should not assume BC provider internals !
+     */
+    public static KeyAgreement getKeyAgreement(final String algorithm) throws NoSuchAlgorithmException {
+        try {
+            final Provider provider = getSecurityProviderIfAccessible();
+            if ( provider != null ) return getKeyAgreement(algorithm, provider);
+        }
+        catch (NoSuchAlgorithmException e) { }
+        catch (SecurityException e) { debugStackTrace(e); }
+        return KeyAgreement.getInstance(algorithm);
+    }
+
+    static KeyAgreement getKeyAgreement(final String algorithm, final Provider provider)
+        throws NoSuchAlgorithmException {
+        final KeyAgreementSpi spi = (KeyAgreementSpi) getImplEngine("KeyAgreement", algorithm);
+        if ( spi == null ) throw new NoSuchAlgorithmException(algorithm + " not found");
+
+        return newInstance(KeyAgreement.class,
+            new Class[] { KeyAgreementSpi.class, Provider.class, String.class },
+            new Object[] { spi, provider, algorithm }
+        );
+    }
+
     /**
      * @note code calling this should not assume BC provider internals !
      */
     public static SecretKeyFactory getSecretKeyFactory(final String algorithm) throws NoSuchAlgorithmException {
         try {
-            final Provider provider = getSecurityProvider();
+            final Provider provider = getSecurityProviderIfAccessible();
             if ( provider != null ) return getSecretKeyFactory(algorithm, provider);
         }
         catch (NoSuchAlgorithmException e) { }
@@ -571,7 +644,7 @@ public abstract class SecurityHelper {
         throws NoSuchAlgorithmException {
         try {
             if ( providerSSLContext ) {
-                final Provider provider = getSecurityProvider();
+                final Provider provider = getSecurityProviderIfAccessible();
                 if ( provider != null ) {
                     return getSSLContext(protocol, provider);
                 }
