@@ -151,13 +151,23 @@ public class X509Cert extends RubyObject {
         return wrap(runtime.getCurrentContext(), cert.getEncoded());
     }
 
+    static X509Cert wrap(ThreadContext context, Certificate cert)
+        throws CertificateEncodingException {
+        return wrap(context, cert.getEncoded());
+    }
+
     // this is the javax.security counterpart of the previous wrap method
     public static IRubyObject wrap(Ruby runtime, javax.security.cert.Certificate cert)
         throws javax.security.cert.CertificateEncodingException {
         return wrap(runtime.getCurrentContext(), cert.getEncoded());
     }
 
-    static IRubyObject wrap(final ThreadContext context, final byte[] encoded) {
+    static X509Cert wrap(ThreadContext context, javax.security.cert.Certificate cert)
+        throws javax.security.cert.CertificateEncodingException {
+        return wrap(context, cert.getEncoded());
+    }
+
+    static X509Cert wrap(final ThreadContext context, final byte[] encoded) {
         //final Ruby runtime = context.runtime;
         //final RubyString enc = StringHelper.newString(runtime, encoded);
         //return _Certificate(runtime).callMethod(context, "new", enc);
@@ -310,11 +320,23 @@ public class X509Cert extends RubyObject {
 
         text.append("Certificate:\n");
         text.append(S20,0,4).append("Data:\n");
-        final int version = RubyNumeric.fix2int(this.version);
+        final int version = this.version == null ? 0 : RubyNumeric.fix2int(this.version);
         text.append(S20,0,8).append("Version: ").append( version + 1 ).
              append(" (0x").append( Integer.toString( version, 16 ) ).append(")\n");
-        text.append(S20,0,8).append("Serial Number:\n");
-        text.append(S20,0,12).append( lowerHexBytes(serial.toByteArray(), 1) ).append('\n');
+        // <= 0x1122334455667788 printed on same line as :
+        // Serial Number: 1234605616436508552 (0x1122334455667788)
+        // but 0x112233445566778899 ends up :
+        // Serial Number:
+        //      11:22:33:44:55:66:77:88:99
+        text.append(S20,0,8).append("Serial Number:");
+        if ( serial.compareTo( new BigInteger("FFFFFFFFFFFFFFFF", 16) ) > 0 ) {
+            text.append('\n');
+            text.append(S20,0,12).append( lowerHexBytes(serial.toByteArray(), 1) ).append('\n');
+        }
+        else {
+            text.append(' ').append(serial.toString(10)).append(' ');
+            text.append('(').append("0x").append(serial.toString(16)).append(')').append('\n');
+        }
 
         text.append(S20,0,4).append("Signature Algorithm: ").append( signature_algorithm() ).append('\n');
         //final RubyString issuer = issuer().asString(); ByteList bytes = issuer.getByteList();
@@ -382,12 +404,12 @@ public class X509Cert extends RubyObject {
 
     @JRubyMethod
     public IRubyObject version() {
-        return version;
+        return version != null ? version : ( version = getRuntime().newFixnum(0) );
     }
 
     @JRubyMethod(name = "version=")
     public IRubyObject set_version(final IRubyObject version) {
-        if ( ! version.equals(this.version) ) {
+        if ( ! version().equals(version) ) {
             this.changed = true;
         }
         return this.version = version;
@@ -402,6 +424,8 @@ public class X509Cert extends RubyObject {
         return cert.getSignature();
     }
 
+    BigInteger getSerial() { return serial; }
+
     @JRubyMethod
     public IRubyObject serial() {
         return BN.newBN(getRuntime(), serial);
@@ -409,21 +433,20 @@ public class X509Cert extends RubyObject {
 
     @JRubyMethod(name = "serial=")
     public IRubyObject set_serial(final IRubyObject serial) {
-        if ( ! serial.equals(this.serial) ) {
-            this.changed = true;
-        }
-
-        final String serialStr = serial.toString();
+        final String serialStr = serial.asString().toString();
         final BigInteger serialInt;
         if ( serialStr.equals("0") ) { // MRI compatibility: allow 0 serial number
             serialInt = BigInteger.ONE;
         } else {
             serialInt = new BigInteger(serialStr);
         }
+        this.changed = ! serialInt.equals(this.serial);
         //generator.setSerialNumber( serialInt.abs() );
         this.serial = serialInt; return serial;
     }
 
+    X509Name getSubject() { return ((X509Name) subject); }
+
     @JRubyMethod
     public IRubyObject subject() {
         return subject;
@@ -435,6 +458,8 @@ public class X509Cert extends RubyObject {
         return this.subject = subject;
     }
 
+    X509Name getIssuer() { return ((X509Name) issuer); }
+
     @JRubyMethod
     public IRubyObject issuer() {
         return issuer;
@@ -542,8 +567,14 @@ public class X509Cert extends RubyObject {
 
         // Have to obey some artificial constraints of the OpenSSL implementation. Stupid.
         final String keyAlg = ((PKey) key).getAlgorithm();
-        final String digAlg = ((Digest) digest).getShortAlgorithm();
-        final String digName = ((Digest) digest).name().toString();
+        final String digAlg; final String digName;
+        if (digest instanceof Digest) {
+            digAlg = ((Digest) digest).getShortAlgorithm();
+            digName = ((Digest) digest).name().toString();
+        }
+        else {
+            digAlg = digest.asJavaString(); digName = null;
+        }
 
         if( ( "DSA".equalsIgnoreCase(keyAlg) && "MD5".equalsIgnoreCase(digAlg) ) ||
             ( "RSA".equalsIgnoreCase(keyAlg) && "DSS1".equals(digName) ) ) {
@@ -728,4 +759,13 @@ public class X509Cert extends RubyObject {
         return GeneralNames.getInstance( value ).getNames();
     }
 
+    @Override
+    public Object toJava(Class target) {
+        if ( target.isAssignableFrom(X509Certificate.class) ) {
+            if ( target == X509AuxCertificate.class ) return getAuxCert();
+            return cert;
+        }
+        return super.toJava(target);
+    }
+
 }// X509Cert
