@@ -34,8 +34,6 @@ import java.io.BufferedWriter;
 import java.io.BufferedReader;
 import java.io.Reader;
 import java.io.ByteArrayInputStream;
-import java.io.ByteArrayOutputStream;
-
 import java.math.BigInteger;
 
 import java.security.GeneralSecurityException;
@@ -54,8 +52,11 @@ import java.security.cert.X509Certificate;
 import java.security.cert.X509CRL;
 import java.security.interfaces.DSAPublicKey;
 import java.security.interfaces.DSAPrivateKey;
+import java.security.interfaces.ECPrivateKey;
+import java.security.interfaces.ECPublicKey;
 import java.security.interfaces.RSAPublicKey;
 import java.security.interfaces.RSAPrivateCrtKey;
+import java.security.spec.ECParameterSpec;
 import java.security.spec.InvalidKeySpecException;
 import java.security.spec.KeySpec;
 import java.security.spec.PKCS8EncodedKeySpec;
@@ -127,8 +128,8 @@ import org.jruby.ext.openssl.Cipher.Algorithm;
 import org.jruby.ext.openssl.impl.ASN1Registry;
 import org.jruby.ext.openssl.impl.CipherSpec;
 import org.jruby.ext.openssl.impl.PKCS10Request;
-
 import org.jruby.ext.openssl.SecurityHelper;
+import org.jruby.ext.openssl.util.ByteArrayOutputStream;
 
 /**
  * Helper class to read and write PEM files correctly.
@@ -205,22 +206,25 @@ public class PEMInputOutput {
             if ( line.indexOf(BEG_STRING_PUBLIC) != -1 ) {
                 try {
                     return readPublicKey(reader,BEF_E+PEM_STRING_PUBLIC);
-                } catch (Exception e) {
-                    throw new IOException("problem creating public key: " + e.toString(), e);
+                }
+                catch (Exception e) {
+                    throw mapReadException("problem creating public key: ", e);
                 }
             }
             else if ( line.indexOf(BEG_STRING_DSA) != -1 ) {
                 try {
                     return readKeyPair(reader,passwd, "DSA", BEF_E+PEM_STRING_DSA);
-                } catch (Exception e) {
-                    throw new IOException("problem creating DSA private key: " + e.toString(), e);
+                }
+                catch (Exception e) {
+                    throw mapReadException("problem creating DSA private key: ", e);
                 }
             }
             else if ( line.indexOf(BEG_STRING_RSA_PUBLIC) != -1 ) {
                 try {
                     return readPublicKey(reader,BEF_E+PEM_STRING_RSA_PUBLIC);
-                } catch (Exception e) {
-                    throw new IOException("problem creating RSA public key: " + e.toString(), e);
+                }
+                catch (Exception e) {
+                    throw mapReadException("problem creating RSA public key: ", e);
                 }
             }
             else if ( line.indexOf(BEG_STRING_X509_OLD) != -1 ) {
@@ -308,7 +312,8 @@ public class PEMInputOutput {
      * c: PEM_read_PrivateKey + PEM_read_bio_PrivateKey
      * CAUTION: KeyPair#getPublic() may be null.
      */
-    public static KeyPair readPrivateKey(final Reader in, char[] passwd) throws IOException {
+    public static KeyPair readPrivateKey(final Reader in, char[] passwd)
+        throws PasswordRequiredException, IOException {
         final String BEG_STRING_ECPRIVATEKEY = BEF_G + PEM_STRING_ECPRIVATEKEY;
         final String BEG_STRING_PKCS8INF = BEF_G + PEM_STRING_PKCS8INF;
         final String BEG_STRING_PKCS8 = BEF_G + PEM_STRING_PKCS8;
@@ -318,19 +323,26 @@ public class PEMInputOutput {
             if ( line.indexOf(BEG_STRING_RSA) != -1 ) {
                 try {
                     return readKeyPair(reader, passwd, "RSA", BEF_E + PEM_STRING_RSA);
-                } catch (Exception e) {
-                    throw new IOException("problem creating RSA private key: " + e.toString(), e);
+                }
+                catch (Exception e) {
+                    throw mapReadException("problem creating RSA private key: ", e);
                 }
             }
             else if ( line.indexOf(BEG_STRING_DSA) != -1 ) {
                 try {
                     return readKeyPair(reader, passwd, "DSA", BEF_E + PEM_STRING_DSA);
-                } catch (Exception e) {
-                    throw new IOException("problem creating DSA private key: " + e.toString(), e);
+                }
+                catch (Exception e) {
+                    throw mapReadException("problem creating DSA private key: ", e);
                 }
             }
-            else if ( line.indexOf(BEG_STRING_ECPRIVATEKEY) != -1) { // TODO EC!
-                throw new UnsupportedOperationException("EC private key not supported");
+            else if ( line.indexOf(BEG_STRING_ECPRIVATEKEY) != -1) {
+                try {
+                    return readKeyPair(reader, passwd, "ECDSA", BEF_E + PEM_STRING_ECPRIVATEKEY);
+                }
+                catch (Exception e) {
+                    throw mapReadException("problem creating DSA private key: ", e);
+                }
             }
             else if ( line.indexOf(BEG_STRING_PKCS8INF) != -1) {
                 try {
@@ -340,7 +352,7 @@ public class PEMInputOutput {
                     return org.jruby.ext.openssl.impl.PKey.readPrivateKey(((ASN1Object) info.parsePrivateKey()).getEncoded(ASN1Encoding.DER), type);
                 }
                 catch (Exception e) {
-                    throw new IOException("problem creating private key: " + e.toString(), e);
+                    throw mapReadException("problem creating private key: ", e);
                 }
             }
             else if ( line.indexOf(BEG_STRING_PKCS8) != -1 ) {
@@ -355,14 +367,22 @@ public class PEMInputOutput {
                         privKey = derivePrivateKeyPBES1(eIn, algId, passwd);
                     }
                     return new KeyPair(null, privKey);
-                } catch (Exception e) {
-                    throw new IOException("problem creating private key: " + e.toString(), e);
+                }
+                catch (Exception e) {
+                    throw mapReadException("problem creating private key: ", e);
                 }
             }
         }
         return null;
     }
 
+    private static IOException mapReadException(final String message, final Exception ex) {
+        if ( ex instanceof PasswordRequiredException ) {
+            return (PasswordRequiredException) ex;
+        }
+        return new IOException(message + ex, ex);
+    }
+
     private static PrivateKey derivePrivateKeyPBES1(EncryptedPrivateKeyInfo eIn, AlgorithmIdentifier algId, char[] password)
             throws GeneralSecurityException, IOException {
         // From BC's PEMReader
@@ -434,9 +454,8 @@ public class PEMInputOutput {
     // PEM_read_bio_PUBKEY
     public static PublicKey readPubKey(Reader in) throws IOException {
         PublicKey pubKey = readRSAPubKey(in);
-        if (pubKey == null) {
-            pubKey = readDSAPubKey(in);
-        }
+        if (pubKey == null) pubKey = readDSAPubKey(in);
+        if (pubKey == null) pubKey = readECPubKey(in);
         return pubKey;
     }
 
@@ -452,7 +471,7 @@ public class PEMInputOutput {
                     return (DSAPublicKey) readPublicKey(reader, "DSA", BEF_E + PEM_STRING_DSA_PUBLIC);
                 }
                 catch (Exception e) {
-                    throw new IOException("problem creating DSA public key: " + e.toString(), e);
+                    throw mapReadException("problem creating DSA public key: ", e);
                 }
             }
         }
@@ -470,7 +489,7 @@ public class PEMInputOutput {
                     return (DSAPublicKey) readPublicKey(reader, "DSA", BEF_E + PEM_STRING_PUBLIC);
                 }
                 catch (Exception e) {
-                    throw new IOException("problem creating DSA public key: " + e.toString(), e);
+                    throw mapReadException("problem creating DSA public key: ", e);
                 }
             }
         }
@@ -480,7 +499,8 @@ public class PEMInputOutput {
     /*
      * c: PEM_read_bio_DSAPrivateKey
      */
-    public static KeyPair readDSAPrivateKey(final Reader in, final char[] passwd) throws IOException {
+    public static KeyPair readDSAPrivateKey(final Reader in, final char[] passwd)
+        throws PasswordRequiredException, IOException {
         final BufferedReader reader = makeBuffered(in); String line;
         while ( ( line = reader.readLine() ) != null ) {
             if ( line.indexOf(BEG_STRING_DSA) != -1 ) {
@@ -488,7 +508,7 @@ public class PEMInputOutput {
                     return readKeyPair(reader, passwd, "DSA", BEF_E + PEM_STRING_DSA);
                 }
                 catch (Exception e) {
-                    throw new IOException("problem creating DSA private key: " + e.toString(), e);
+                    throw mapReadException("problem creating DSA private key: ", e);
                 }
             }
         }
@@ -505,15 +525,17 @@ public class PEMInputOutput {
             if ( line.indexOf(BEG_STRING_PUBLIC) != -1 ) {
                 try {
                     return readRSAPublicKey(reader, BEF_E + PEM_STRING_PUBLIC);
-                } catch (Exception e) {
-                    throw new IOException("problem creating RSA public key: " + e.toString(), e);
+                }
+                catch (Exception e) {
+                    throw mapReadException("problem creating RSA public key: ", e);
                 }
             }
             else if ( line.indexOf(BEG_STRING_RSA_PUBLIC) != -1 ) {
                 try {
                     return readRSAPublicKey(reader, BEF_E + PEM_STRING_RSA_PUBLIC);
-                } catch (Exception e) {
-                    throw new IOException("problem creating RSA public key: " + e.toString(), e);
+                }
+                catch (Exception e) {
+                    throw mapReadException("problem creating RSA public key: ", e);
                 }
             }
         }
@@ -524,7 +546,8 @@ public class PEMInputOutput {
      * reads an RSA public key encoded in an PKCS#1 RSA structure.
      * c: PEM_read_bio_RSAPublicKey
      */
-    public static RSAPublicKey readRSAPublicKey(Reader in, char[] f) throws IOException {
+    public static RSAPublicKey readRSAPublicKey(Reader in, char[] f)
+        throws PasswordRequiredException, IOException {
         final BufferedReader reader = makeBuffered(in); String line;
         while ( ( line = reader.readLine() ) != null ) {
             if ( line.indexOf(BEG_STRING_PUBLIC) != -1 ) {
@@ -532,7 +555,7 @@ public class PEMInputOutput {
                     return (RSAPublicKey) readPublicKey(reader, "RSA", BEF_E + PEM_STRING_PUBLIC);
                 }
                 catch (Exception e) {
-                    throw new IOException("problem creating RSA public key: " + e.toString(), e);
+                    throw mapReadException("problem creating RSA public key: ", e);
                 }
             }
             else if ( line.indexOf(BEF_G+PEM_STRING_RSA_PUBLIC) != -1 ) {
@@ -540,7 +563,7 @@ public class PEMInputOutput {
                     return (RSAPublicKey) readPublicKey(reader, "RSA", BEF_E + PEM_STRING_RSA_PUBLIC);
                 }
                 catch (Exception e) {
-                    throw new IOException("problem creating RSA public key: " + e.toString(), e);
+                    throw mapReadException("problem creating RSA public key: ", e);
                 }
             }
         }
@@ -550,7 +573,8 @@ public class PEMInputOutput {
     /**
      * c: PEM_read_bio_RSAPrivateKey
      */
-    public static KeyPair readRSAPrivateKey(Reader in, char[] f) throws IOException {
+    public static KeyPair readRSAPrivateKey(Reader in, char[] f)
+        throws PasswordRequiredException, IOException {
         final BufferedReader reader = makeBuffered(in); String line;
         while ( ( line = reader.readLine() ) != null ) {
             if ( line.indexOf(BEG_STRING_RSA) != -1 ) {
@@ -558,7 +582,56 @@ public class PEMInputOutput {
                     return readKeyPair(reader,f, "RSA", BEF_E + PEM_STRING_RSA);
                 }
                 catch (Exception e) {
-                    throw new IOException("problem creating RSA private key: " + e.toString(), e);
+                    throw mapReadException("problem creating RSA private key: ", e);
+                }
+            }
+        }
+        return null;
+    }
+
+    public static ECPublicKey readECPubKey(Reader in) throws IOException {
+        final String BEG_STRING_EC_PUBLIC = BEF_G + "EC PUBLIC KEY";
+        final BufferedReader reader = makeBuffered(in); String line;
+        while ( ( line = reader.readLine() ) != null ) {
+            if ( line.indexOf(BEG_STRING_EC_PUBLIC) != -1 ) {
+                try {
+                    return (ECPublicKey) readPublicKey(reader, "ECDSA", BEF_E + "EC PUBLIC KEY");
+                }
+                catch (Exception e) {
+                    throw mapReadException("problem creating ECDSA public key: ", e);
+                }
+            }
+        }
+        return null;
+    }
+
+    public static ECPublicKey readECPublicKey(final Reader in, final char[] passwd) throws IOException {
+        // final String BEG_STRING_EC = BEF_G + "EC PUBLIC KEY";
+        final BufferedReader reader = makeBuffered(in); String line;
+        while ( ( line = reader.readLine() ) != null ) {
+            if ( line.indexOf(BEG_STRING_PUBLIC) != -1 ) {
+                try {
+                    return (ECPublicKey) readPublicKey(reader, "ECDSA", BEF_E + PEM_STRING_PUBLIC);
+                }
+                catch (Exception e) {
+                    throw mapReadException("problem creating ECDSA public key: ", e);
+                }
+            }
+        }
+        return null;
+    }
+
+    public static KeyPair readECPrivateKey(final Reader in, final char[] passwd)
+        throws PasswordRequiredException, IOException {
+        final String BEG_STRING_EC = BEF_G + "EC PRIVATE KEY";
+        final BufferedReader reader = makeBuffered(in); String line;
+        while ( ( line = reader.readLine() ) != null ) {
+            if ( line.indexOf(BEG_STRING_EC) != -1 ) {
+                try {
+                    return readKeyPair(reader, passwd, "ECDSA", BEF_E + "EC PRIVATE KEY");
+                }
+                catch (Exception e) {
+                    throw mapReadException("problem creating ECDSA private key: ", e);
                 }
             }
         }
@@ -771,70 +844,83 @@ public class PEMInputOutput {
 
     public static void writeDSAPublicKey(Writer _out, DSAPublicKey obj) throws IOException {
         BufferedWriter out = makeBuffered(_out);
-        byte[] encoding = getEncoded(obj);
-        out.write(BEF_G + PEM_STRING_DSA_PUBLIC + AFT);
+        final byte[] enc = getEncoded(obj);
+        out.write(BEF_G + PEM_STRING_PUBLIC + AFT);
         out.newLine();
-        writeEncoded(out, encoding);
-        out.write(BEF_E + PEM_STRING_DSA_PUBLIC + AFT);
+        writeEncoded(out, enc, enc.length);
+        out.write(BEF_E + PEM_STRING_PUBLIC + AFT);
         out.newLine();
         out.flush();
     }
+
     /** writes an RSA public key encoded in an PKCS#1 RSA structure. */
     public static void writeRSAPublicKey(Writer _out, RSAPublicKey obj) throws IOException {
         BufferedWriter out = makeBuffered(_out);
-        byte[] encoding = getEncoded(obj);
-        out.write(BEF_G + PEM_STRING_RSA_PUBLIC + AFT);
+        final byte[] enc = getEncoded(obj);
+        out.write(BEF_G + PEM_STRING_PUBLIC + AFT);
         out.newLine();
-        writeEncoded(out, encoding);
-        out.write(BEF_E + PEM_STRING_RSA_PUBLIC + AFT);
+        writeEncoded(out, enc, enc.length);
+        out.write(BEF_E + PEM_STRING_PUBLIC + AFT);
         out.newLine();
         out.flush();
     }
+
+    public static void writeECPublicKey(Writer _out, ECPublicKey obj) throws IOException {
+        BufferedWriter out = makeBuffered(_out);
+        final byte[] enc = getEncoded(obj);
+        out.write(BEF_G); out.write(PEM_STRING_PUBLIC); out.write(AFT);
+        out.newLine();
+        writeEncoded(out, enc, enc.length);
+        out.write(BEF_E); out.write(PEM_STRING_PUBLIC); out.write(AFT);
+        out.newLine();
+        out.flush();
+    }
+
     public static void writePKCS7(Writer _out, ContentInfo obj) throws IOException {
         BufferedWriter out = makeBuffered(_out);
-        byte[] encoding = getEncoded(obj);
+        final byte[] enc = getEncoded(obj);
         out.write(BEF_G + PEM_STRING_PKCS7 + AFT);
         out.newLine();
-        writeEncoded(out,encoding);
+        writeEncoded(out, enc, enc.length);
         out.write(BEF_E + PEM_STRING_PKCS7 + AFT);
         out.newLine();
         out.flush();
     }
     public static void writePKCS7(Writer _out, CMSSignedData obj) throws IOException {
         BufferedWriter out = makeBuffered(_out);
-        byte[] encoding = getEncoded(obj);
+        final byte[] enc = getEncoded(obj);
         out.write(BEF_G + PEM_STRING_PKCS7 + AFT);
         out.newLine();
-        writeEncoded(out,encoding);
+        writeEncoded(out, enc, enc.length);
         out.write(BEF_E + PEM_STRING_PKCS7 + AFT);
         out.newLine();
         out.flush();
     }
-    public static void writePKCS7(final Writer _out, final byte[] encoded) throws IOException {
+    public static void writePKCS7(final Writer _out, final byte[] enc) throws IOException {
         BufferedWriter out = makeBuffered(_out);
         out.write(BEF_G + PEM_STRING_PKCS7 + AFT);
         out.newLine();
-        writeEncoded(out,encoded);
+        writeEncoded(out, enc, enc.length);
         out.write(BEF_E + PEM_STRING_PKCS7 + AFT);
         out.newLine();
         out.flush();
     }
     public static void writeX509Certificate(final Writer _out, final X509Certificate cert) throws IOException {
         BufferedWriter out = makeBuffered(_out);
-        byte[] encoding = getEncoded(cert);
+        final byte[] enc = getEncoded(cert);
         out.write(BEF_G + PEM_STRING_X509 + AFT);
         out.newLine();
-        writeEncoded(out, encoding);
+        writeEncoded(out, enc, enc.length);
         out.write(BEF_E + PEM_STRING_X509 + AFT);
         out.newLine();
         out.flush();
     }
     public static void writeX509Aux(final Writer _out, final X509AuxCertificate cert) throws IOException {
         BufferedWriter out = makeBuffered(_out);
-        byte[] encoding;
+        final byte[] encoding; final int encLen;
         try {
             if ( cert.aux == null ) {
-                encoding = cert.getEncoded();
+                encoding = cert.getEncoded(); encLen = encoding.length;
             }
             else {
                 ByteArrayOutputStream baos = new ByteArrayOutputStream();
@@ -870,7 +956,7 @@ public class PEMInputOutput {
                 }
                 enc = new DLSequence(a1).getEncoded();
                 baos.write(enc, 0, enc.length);
-                encoding = baos.toByteArray();
+                encoding = baos.buffer(); encLen = baos.size();
             }
         }
         catch (CertificateEncodingException e) {
@@ -878,7 +964,7 @@ public class PEMInputOutput {
         }
         out.write(BEF_G + PEM_STRING_X509_TRUSTED + AFT);
         out.newLine();
-        writeEncoded(out,encoding);
+        writeEncoded(out, encoding, encLen);
         out.write(BEF_E + PEM_STRING_X509_TRUSTED + AFT);
         out.newLine();
         out.flush();
@@ -888,7 +974,7 @@ public class PEMInputOutput {
         byte[] encoding = getEncoded(obj);
         out.write(BEF_G + PEM_STRING_X509_CRL + AFT);
         out.newLine();
-        writeEncoded(out, encoding);
+        writeEncoded(out, encoding, encoding.length);
         out.write(BEF_E + PEM_STRING_X509_CRL + AFT);
         out.newLine();
         out.flush();
@@ -898,7 +984,7 @@ public class PEMInputOutput {
         byte[] encoding = getEncoded(obj.toASN1Structure());
         out.write(BEF_G + PEM_STRING_X509_REQ + AFT);
         out.newLine();
-        writeEncoded(out,encoding);
+        writeEncoded(out, encoding, encoding.length);
         out.write(BEF_E + PEM_STRING_X509_REQ + AFT);
         out.newLine();
         out.flush();
@@ -924,12 +1010,11 @@ public class PEMInputOutput {
         v.add(new ASN1Integer(x));
 
         aOut.writeObject(new DLSequence(v));
-        byte[] encoding = bOut.toByteArray();
 
         if (cipher != null && passwd != null) {
-            writePemEncrypted(out, PEM_STRING_DSA, encoding, cipher, passwd);
+            writePemEncrypted(out, PEM_STRING_DSA, bOut.buffer(), bOut.size(), cipher, passwd);
         } else {
-            writePemPlain(out, PEM_STRING_DSA, encoding);
+            writePemPlain(out, PEM_STRING_DSA, bOut.buffer(), bOut.size());
         }
     }
 
@@ -946,17 +1031,53 @@ public class PEMInputOutput {
         }
     }
 
-    private static void writePemPlain(BufferedWriter out, String pemHeader, byte[] encoding) throws IOException {
-        out.write(BEF_G + pemHeader + AFT);
+    public static void writeECPrivateKey(Writer _out, ECPrivateKey obj, CipherSpec cipher, char[] passwd) throws IOException {
+        assert (obj != null);
+        final String PEM_STRING_EC = "EC PRIVATE KEY";
+        BufferedWriter out = makeBuffered(_out);
+        final int bitLength = obj.getParams().getOrder().bitLength();
+        org.bouncycastle.asn1.sec.ECPrivateKey keyStruct = new org.bouncycastle.asn1.sec.ECPrivateKey(bitLength, obj.getS());
+        if (cipher != null && passwd != null) {
+            writePemEncrypted(out, PEM_STRING_EC, keyStruct.getEncoded(), cipher, passwd);
+        } else {
+            writePemPlain(out, PEM_STRING_EC, keyStruct.getEncoded());
+        }
+    }
+
+    public static void writeECParameters(Writer _out, ASN1ObjectIdentifier obj, CipherSpec cipher, char[] passwd) throws IOException {
+        assert (obj != null);
+        final String PEM_STRING_EC = "EC PARAMETERS";
+        BufferedWriter out = makeBuffered(_out);
+        if (cipher != null && passwd != null) {
+            writePemEncrypted(out, PEM_STRING_EC, obj.getEncoded(), cipher, passwd);
+        } else {
+            writePemPlain(out, PEM_STRING_EC, obj.getEncoded());
+        }
+    }
+
+    private static void writePemPlain(final BufferedWriter out,
+        final String PEM_ID, final byte[] encoding) throws IOException {
+        writePemPlain(out, PEM_ID, encoding, encoding.length);
+    }
+
+    private static void writePemPlain(final BufferedWriter out,
+        final String PEM_ID, final byte[] encoding, final int encLen) throws IOException {
+        out.write(BEF_G); out.write(PEM_ID); out.write(AFT);
         out.newLine();
-        writeEncoded(out, encoding);
-        out.write(BEF_E + pemHeader + AFT);
+        writeEncoded(out, encoding, encLen);
+        out.write(BEF_E); out.write(PEM_ID); out.write(AFT);
         out.newLine();
         out.flush();
     }
 
     private static void writePemEncrypted(final BufferedWriter out,
-        final String pemHeader, final byte[] encoding,
+        final String PEM_ID, final byte[] encoding,
+        final CipherSpec cipherSpec, final char[] passwd) throws IOException {
+        writePemEncrypted(out, PEM_ID, encoding, encoding.length, cipherSpec, passwd);
+    }
+
+    private static void writePemEncrypted(final BufferedWriter out,
+        final String PEM_ID, final byte[] encoding, final int encCount,
         final CipherSpec cipherSpec, final char[] passwd) throws IOException {
 
         final Cipher cipher = cipherSpec.getCipher();
@@ -972,7 +1093,7 @@ public class PEMInputOutput {
         final byte[] encData;
         try {
             cipher.init(Cipher.ENCRYPT_MODE, secretKey, new IvParameterSpec(iv));
-            encData = cipher.doFinal(encoding);
+            encData = cipher.doFinal(encoding, 0, encCount);
         }
         catch (InvalidKeyException e) {
             final String msg = e.getMessage();
@@ -984,16 +1105,16 @@ public class PEMInputOutput {
         catch (GeneralSecurityException e) {
             throw new IOException("exception using cipher: "+ cipherSpec.getOsslName()  + " (" + e + ")", e);
         }
-        out.write(BEF_G + pemHeader + AFT);
+        out.write(BEF_G); out.write(PEM_ID); out.write(AFT);
         out.newLine();
         out.write("Proc-Type: 4,ENCRYPTED");
         out.newLine();
-        out.write("DEK-Info: " + cipherSpec.getOsslName() + ",");
+        out.write("DEK-Info: " + cipherSpec.getOsslName() + ',');
         writeHexEncoded(out, iv);
         out.newLine();
         out.newLine();
-        writeEncoded(out, encData);
-        out.write(BEF_E + pemHeader + AFT);
+        writeEncoded(out, encData, encData.length);
+        out.write(BEF_E); out.write(PEM_ID); out.write(AFT);
         out.flush();
     }
 
@@ -1028,11 +1149,10 @@ public class PEMInputOutput {
         ASN1OutputStream aOut = new ASN1OutputStream(bOut);
 
         aOut.writeObject(new DLSequence(v));
-        byte[] encoding = bOut.toByteArray();
 
         out.write(BEF_G); out.write(PEM_STRING_DHPARAMS); out.write(AFT);
         out.newLine();
-        writeEncoded(out, encoding);
+        writeEncoded(out, bOut.buffer(), bOut.size());
         out.write(BEF_E); out.write(PEM_STRING_DHPARAMS); out.write(AFT);
         out.newLine();
         out.flush();
@@ -1078,7 +1198,7 @@ public class PEMInputOutput {
 
     private static PublicKey readPublicKey(BufferedReader in, String endMarker) throws IOException {
         byte[] input = readBase64Bytes(in, endMarker);
-        String[] algs = { "RSA", "DSA" };
+        String[] algs = { "RSA", "DSA", "ECDSA" };
         for (int i = 0; i < algs.length; i++) {
             PublicKey key = readPublicKey(input, algs[i], endMarker);
             if (key != null) {
@@ -1091,7 +1211,8 @@ public class PEMInputOutput {
     /**
      * Read a Key Pair
      */
-    private static KeyPair readKeyPair(BufferedReader in, char[] passwd, String type, String endMarker) throws Exception {
+    private static KeyPair readKeyPair(BufferedReader in, char[] passwd, String type, String endMarker)
+        throws PasswordRequiredException, IOException, GeneralSecurityException {
         boolean isEncrypted = false;
         String dekInfo = null;
 
@@ -1121,10 +1242,10 @@ public class PEMInputOutput {
         return org.jruby.ext.openssl.impl.PKey.readPrivateKey(keyBytes, type);
     }
 
-    private static byte[] decrypt(byte[] decoded, String dekInfo, char[] passwd) throws IOException, GeneralSecurityException {
-        if (passwd == null) {
-            throw new IOException("Password is null, but a password is required");
-        }
+    private static byte[] decrypt(byte[] decoded, String dekInfo, char[] passwd)
+        throws PasswordRequiredException, IOException, GeneralSecurityException {
+        if ( passwd == null ) throw new PasswordRequiredException();
+
         StringTokenizer tknz = new StringTokenizer(dekInfo, ",");
         String algorithm = tknz.nextToken();
         byte[] iv = Hex.decode(tknz.nextToken());
@@ -1151,6 +1272,14 @@ public class PEMInputOutput {
         return cipher.doFinal(decoded);
     }
 
+    public static class PasswordRequiredException extends IOException {
+
+        PasswordRequiredException() {
+            super();
+        }
+
+    }
+
     /**
      * Reads in a X509Certificate.
      *
@@ -1361,9 +1490,10 @@ public class PEMInputOutput {
         }
     }
 
-    private static void writeEncoded(BufferedWriter out, byte[] bytes) throws IOException {
-        char[]  buf = new char[64];
-        bytes = Base64.encode(bytes);
+    private static void writeEncoded(BufferedWriter out,
+        byte[] bytes, final int bytesLen) throws IOException {
+        final char[] buf = new char[64];
+        bytes = Base64.encode(bytes, 0 ,bytesLen);
         for (int i = 0; i < bytes.length; i += buf.length) {
             int index = 0;
 
@@ -1371,7 +1501,7 @@ public class PEMInputOutput {
                 if ((i + index) >= bytes.length) {
                     break;
                 }
-                buf[index] = (char)bytes[i + index];
+                buf[index] = (char) bytes[i + index];
                 index++;
             }
             out.write(buf, 0, index);
