@@ -32,6 +32,7 @@ import static org.jruby.ext.openssl.x509store.X509Utils.X509_FILETYPE_PEM;
 import static org.jruby.ext.openssl.x509store.X509Utils.X509_R_CERT_ALREADY_IN_HASH_TABLE;
 
 import java.io.FileNotFoundException;
+import java.io.IOException;
 import java.security.cert.X509Certificate;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -112,7 +113,7 @@ public class Store implements X509TrustManager {
         };
     }
 
-    @Deprecated int cache = 1; // not-used
+    // @Deprecated int cache = 1; // not-used
 
     private volatile X509Object[] objects = new X509Object[0];
     private volatile Lookup[] certLookups = new Lookup[0];
@@ -372,6 +373,14 @@ public class Store implements X509TrustManager {
         catch (FileNotFoundException e) {
             // set_default_paths ignores FileNotFound
         }
+        catch (IOException e) {
+            // this is for older jrubies as they do not have a
+            // org.jruby.util.ResourceException.NotFound
+            if (!e.getClass().getSimpleName().equals("NotFound")) {
+                throw e;
+            }
+            // set_default_paths ignores FileNotFound
+        }
 
         lookup = addLookup(runtime, Lookup.hashDirLookup());
         //if ( lookup == null ) return 0;
@@ -382,6 +391,12 @@ public class Store implements X509TrustManager {
         catch (FileNotFoundException e) {
             // set_default_paths ignores FileNotFound
         }
+        catch (IOException e) {
+            if (!e.getClass().getSimpleName().equals("NotFound")) {
+                throw e;
+            }
+            // set_default_paths ignores FileNotFound
+        }
 
         X509Error.clearErrors();
         return 1;
