@@ -231,10 +231,7 @@ public abstract class X509Utils {
             if ( sakid.getKeyIdentifier() != null ) {
                 if ( issuer.getExtensionValue("2.5.29.14") != null ) {
                     DEROctetString der = (DEROctetString) get(issuer.getExtensionValue("2.5.29.14"));
-                    if ( der.getOctets().length > 20 ) {
-                        der = (DEROctetString) get(der.getOctets());
-                    }
-                    SubjectKeyIdentifier iskid = SubjectKeyIdentifier.getInstance(der);
+                    SubjectKeyIdentifier iskid = SubjectKeyIdentifier.getInstance(get(der.getOctets()));
                     if ( iskid.getKeyIdentifier() != null ) {
                         if ( ! Arrays.equals( sakid.getKeyIdentifier(), iskid.getKeyIdentifier() ) ) {
                             return V_ERR_AKID_SKID_MISMATCH;
@@ -602,5 +599,15 @@ public abstract class X509Utils {
 
     public static final int EXFLAG_INVALID_POLICY=0x400;
 
+    public static final int XKU_SSL_SERVER=0x1;
+    public static final int XKU_SSL_CLIENT=0x2;
+    public static final int XKU_SMIME=0x4;
+    public static final int XKU_CODE_SIGN=0x8;
+    public static final int XKU_SGC=0x8;
+    public static final int XKU_OCSP_SIGN=0x20;
+    public static final int XKU_TIMESTAMP=0x40;
+    public static final int XKU_DVCS=0x80;
+    public static final int XKU_ANYEKU=0x100;
+
     public static final int POLICY_FLAG_ANY_POLICY = 0x2;
 }// X509
