@@ -68,6 +68,7 @@ class TestCipher < TestCase
     #puts OpenSSL::Cipher.ciphers.size
 
     OpenSSL::Cipher.ciphers.each do |cipher_name|
+      next if cipher_name.end_with?('wrap') # e.g. 'id-aes256-wrap'
       OpenSSL::Cipher.new cipher_name
     end
   end
@@ -312,6 +313,130 @@ class TestCipher < TestCase
     end
   end
 
+  def test_aes_128_gcm
+    cipher = OpenSSL::Cipher.new('aes-128-gcm')
+    assert_equal cipher, cipher.encrypt
+    cipher.key = '01' * 8
+    cipher.iv = '0' * 16
+
+    bytes = '0000' * 4
+    expected = "\xAC\xC8\x0E\xEDbX,\xB4\xCD\x02\x06O(p\xF8u" # from MRI
+    actual = cipher.update(bytes)
+    assert_equal expected, actual
+    assert_equal "", cipher.final unless defined? JRUBY_VERSION
+
+    cipher = OpenSSL::Cipher.new('aes-128-gcm')
+    assert_equal cipher, cipher.encrypt
+    cipher.key = '01' * 8
+    cipher.iv = '012345678' * 2
+
+    bytes = '0000' * 4
+    expected = "\xF3\xEF\xE6K\xBAJ\xAB=7m'\b\xE0\x06U\x9B" # from MRI
+    actual = cipher.update(bytes)
+    assert_equal expected, actual
+    #assert_equal "", cipher.final unless defined? JRUBY_VERSION
+
+    cipher = OpenSSL::Cipher.new('aes-128-gcm')
+    assert_equal cipher, cipher.encrypt
+    assert_equal 16, cipher.key_len
+    assert_equal 12, cipher.iv_len
+    cipher.key = '01' * 8
+    cipher.iv = '0' * 12
+
+    bytes = '0000' * 4
+    expected = "\xAC\xC8\x0E\xEDbX,\xB4\xCD\x02\x06O(p\xF8u" # from MRI
+    actual = cipher.update(bytes)
+    assert_equal expected, actual
+    #assert_equal "", cipher.final
+
+    cipher = OpenSSL::Cipher.new('aes-256-gcm')
+    assert_equal cipher, cipher.encrypt
+    assert_equal 32, cipher.key_len
+    assert_equal 12, cipher.iv_len
+    cipher.key = '01245678' * 4
+    cipher.iv = '0123456' * 2
+
+    bytes = '0101' * 8
+    expected = "\xA8I0\xF8\xCD?Z\xFD\x8E\"T\xF5\xF2\xC5\xC8\x05\xD4b\x85\xA3}'\xC99]\xC1\x16\x8B\x13\x9E-)" # from MRI
+    actual = cipher.update(bytes)
+    assert_equal expected, actual
+    #assert_equal "", cipher.final
+  end
+
+  def test_aes_gcm
+    ['aes-128-gcm', 'aes-192-gcm', 'aes-256-gcm'].each do |algo|
+      pt = "You should all use Authenticated Encryption!"
+      cipher, key, iv = new_encryptor(algo)
+
+      cipher.auth_data = "aad"
+      ct  = cipher.update(pt) + cipher.final
+      tag = cipher.auth_tag
+      assert_equal(16, tag.size)
+
+      decipher = new_decryptor(algo, key, iv)
+      decipher.auth_tag = tag
+      decipher.auth_data = "aad"
+
+      assert_equal(pt, decipher.update(ct) + decipher.final)
+    end
+  end
+
+  def new_encryptor(algo)
+    cipher = OpenSSL::Cipher.new(algo)
+    cipher.encrypt
+    key = cipher.random_key
+    iv = cipher.random_iv
+    [cipher, key, iv]
+  end
+  private :new_encryptor
+
+  def new_decryptor(algo, key, iv)
+    OpenSSL::Cipher.new(algo).tap do |cipher|
+      cipher.decrypt
+      cipher.key = key
+      cipher.iv = iv
+    end
+  end
+  private :new_decryptor
+
+  def test_aes_128_gcm_with_auth_tag
+    cipher = OpenSSL::Cipher.new('aes-128-gcm')
+    cipher.encrypt
+    #assert_equal 16, cipher.key_len
+    #assert_equal 12, cipher.iv_len
+    cipher.key = '01' * 8
+    cipher.iv = '1001' * 3
+
+    plaintext = "Hello World"
+
+    padding = cipher.update("\0\0")
+    text = cipher.update(plaintext)
+
+    final = cipher.final; a_tag = cipher.auth_tag
+
+    assert_equal "\xB5\xFD", padding unless defined? JRUBY_VERSION
+    assert_equal "\xCCxqd\xDE\x92\x95\xAD0\xB4=", text unless defined? JRUBY_VERSION
+    assert_equal "", final unless defined? JRUBY_VERSION
+
+    assert_equal "\xB5\xFD\xCCxqd\xDE\x92\x95\xAD0\xB4=", padding + text + final
+
+    assert_equal "\ay\xBA\x89\xC9\x91\xF8N\xB7\xD6\x17+\x0F\\\xF8N", a_tag
+
+    assert_equal a_tag, cipher.auth_tag
+    assert_raise(OpenSSL::Cipher::CipherError) { cipher.update("\0\0") }
+    assert_equal a_tag, cipher.auth_tag
+    assert_raise(OpenSSL::Cipher::CipherError) { cipher.final }
+  end
+
+  def test_encrypt_auth_data_non_gcm
+    cipher = OpenSSL::Cipher.new 'aes-128-cfb'
+    cipher.encrypt
+    #length = 16
+    #cipher.iv = '0' * length
+    #cipher.key = '1' * length
+    assert_raise(OpenSSL::Cipher::CipherError) { cipher.auth_tag }
+  end
+
   def test_encrypt_aes_cfb_16_incompatibility
     cipher = OpenSSL::Cipher.new 'AES-128-CFB'
     assert_equal cipher, cipher.encrypt
