@@ -8,12 +8,16 @@ if bc_version = ENV['BC_VERSION'] # && respond_to?(:require_jar)
   require_jar 'org.bouncycastle', 'bcprov-jdk15on', bc_version
   Jars.freeze_loading if defined? Jars.freeze_loading
 
-  puts Java::OrgBouncycastleJceProvider::BouncyCastleProvider.new.info
+  puts Java::OrgBouncycastleJceProvider::BouncyCastleProvider.new.info if $VERBOSE
 else
-  $CLASSPATH << 'pkg/classes'
-  jar = Dir['lib/org/bouncycastle/**/bcprov-*.jar'].first
+  base_dir = File.expand_path('../../..', File.dirname(__FILE__))
+
+  jar = File.join(base_dir, 'lib/jopenssl.jar')
+  raise "jopenssl.jar jar not found" unless jar; $CLASSPATH << jar
+
+  jar = Dir[File.join(base_dir, 'lib/org/bouncycastle/**/bcprov-*.jar')].first
   raise "bcprov jar not found" unless jar; $CLASSPATH << jar
-  jar = Dir['lib/org/bouncycastle/**/bcpkix-*.jar'].first
+  jar = Dir[File.join(base_dir, 'lib/org/bouncycastle/**/bcpkix-*.jar')].first
   raise "bcpkix jar not found" unless jar; $CLASSPATH << jar
 end if defined? JRUBY_VERSION
 
@@ -89,21 +93,36 @@ TestCase.class_eval do
     end
   end
 
-  def self.disable_security_restrictions!; end # do nothing on MRI
+  unless method_defined?(:assert_not_same)
+    def assert_not_same(expected, actual)
+      assert ! expected.equal?(actual), "expected: #{expected} to be same as: #{actual} but did"
+    end
+  end
+
+  def self.disable_security_restrictions!; @@security_restrictions = nil end # do nothing on MRI
+
+  @@security_restrictions = ''
 
   def self.disable_security_restrictions!
-    security_class = java.lang.Class.for_name('javax.crypto.JceSecurity')
-    restricted_field = security_class.get_declared_field('isRestricted')
-    restricted_field.accessible = true
-    restricted_field.set nil, false; return true
-  rescue Java::JavaLang::ClassNotFoundException => e
-    warn "failed to disable JCE security restrictions: #{e.inspect}"; nil
-  rescue Java::JavaLang::NoSuchFieldException => e # Java 6
-    warn "failed to disable JCE security restrictions: #{e.inspect}"; nil
-  rescue NameError => e
-    warn "failed to disable JCE security restrictions: #{e.inspect}"; nil
+    debug = OpenSSL.debug
+    begin
+      OpenSSL.debug = true
+      #org.jruby.ext.openssl.util.CryptoSecurity.unrestrictSecurity
+      #org.jruby.ext.openssl.util.CryptoSecurity.setAllPermissionPolicy
+      @@security_restrictions = OpenSSL.send :_disable_security_restrictions!
+    ensure
+      OpenSSL.debug = debug
+    end
   end if defined? JRUBY_VERSION
 
+  def self.disable_security_restrictions
+    disable_security_restrictions! if @@security_restrictions.eql?('')
+  end
+
+  def self.security_restrictions?
+    disable_security_restrictions; return @@security_restrictions
+  end
+
   def self.java6?; java_version.last.to_i == 6 end
   def self.java7?; java_version.last.to_i == 7 end
   def self.java8?; java_version.last.to_i == 8 end
@@ -118,8 +137,7 @@ TestCase.class_eval do
 
   private
 
-  def issue_cert(dn, key, serial, not_before, not_after, extensions,
-                 issuer, issuer_key, digest)
+  def issue_cert(dn, key, serial, not_before, not_after, extensions, issuer, issuer_key, digest)
     cert = OpenSSL::X509::Certificate.new
     issuer = cert unless issuer
     issuer_key = key unless issuer_key
@@ -204,4 +222,4 @@ if ENV['OPENSSL_TEST_SUITE'].to_s == 'true'
   Dir.glob("#{File.join(base, sub)}/**/test_*.rb").each do |test|
     require test
   end
-end
\ No newline at end of file
+end
